/*******************************************************************************
 * Quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Component definitions.
 ******************************************************************************/
const NLF_CLASSNAME = "Netscape News Live Feed";
const NLF_CLASSID = Components.ID("{ef7ee77b-7914-450a-8338-9c88f74fce6a}");
const NLF_CONTRACTID = "@netscape.com/newslive/feed;1";

const NLI_CLASSNAME = "Netscape News Live Item";
const NLI_CLASSID = Components.ID("{3d2b1145-6c85-4c57-a853-480b5d894c02}");
const NLI_CONTRACTID = "@netscape.com/newslive/item;1";

const NLS_CLASSNAME = "Netscape News Live Service";
const NLS_CLASSID = Components.ID("{23d7cf44-7dc7-40cb-becd-3b21a8576461}");
const NLS_CONTRACTID = "@netscape.com/newslive/service;1";
/*******************************************************************************
 * Contract IDs.
 ******************************************************************************/
const BUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
const STORAGE_CONTRACTID = "@mozilla.org/storage/service;1"; 
const WRAPPER_CONTRACTID = "@mozilla.org/storage/statement-wrapper;1";
const ARRAY_CONTRACTID = "@mozilla.org/array;1";
const PREF_CONTRACTID = "@mozilla.org/preferences-service;1";
const DIR_CONTRACTID = "@mozilla.org/file/directory_service;1";
const CAT_CONTRACTID = "@mozilla.org/categorymanager;1";
const IO_CONTRACTID = "@mozilla.org/network/io-service;1";
const OB_CONTRACTID = "@mozilla.org/observer-service;1";
const REQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
const UUID_CONTRACTID = "@mozilla.org/uuid-generator;1";
const FPR_CONTRACTID = "@mozilla.org/feed-processor;1";
const TIMER_CONTRACTID = "@mozilla.org/timer;1";
const LOAD_CONTRACTID = "@mozilla.org/network/load-group;1";
const UNESCAPE_CONTRACTID = "@mozilla.org/feed-unescapehtml;1";
/*******************************************************************************
 * Observer topics
 ******************************************************************************/
const TOPIC_TIMER = "timer-callback";
const TOPIC_DEFAULT = "netscape-newslive";
const TOPIC_SHUTDOWN_APP = "quit-application";
const TOPIC_SHUTDOWN_XPCOM = "xpcom-shutdown";
const TOPIC_STARTUP_APP = "app-startup";
const TOPIC_STARTUP_UI = "final-ui-startup";
const TOPIC_PREF = "nsPref:changed";
/*******************************************************************************
 * Default values used across components.
 ******************************************************************************/
const DEFAULT_BRANCH = "extensions.netscape.newslive.";
const DEFAULT_FAILED = "Failed to Load News";
const DEFAULT_BUNDLE = "chrome://browser/locale/netscape-newslive/newslive.properties";
const DEFAULT_SEC = 1000;                                    // 1 second
const DEFAULT_MIN = 60*DEFAULT_SEC;                          // 1 minute
const DEFAULT_HOUR = 60*DEFAULT_MIN;                         // 1 hour
const DEFAULT_DAY = 24*DEFAULT_HOUR;                         // 1 day
const DEFAULT_TIMER = DEFAULT_SEC;                           // 1 second
/*******************************************************************************
 * FeedsChecker values.
 ******************************************************************************/
const FEEDS_URL = "http://www.netscape.com/api/feeds/";
const FEEDS_TIMER = DEFAULT_HOUR;                            // 1 hour
const FEEDS_EXPIRES = 7*DEFAULT_DAY;                         // 7 day
const FEEDS_EXPIRES_MAX = 60*DEFAULT_DAY;                    // 60 days
const FEEDS_VAR = 30*DEFAULT_MIN;                            // 30 minutes
const FEEDS_SAME = "sameFeeds";
const FEEDS_NEW = "newFeeds";
const FEEDS_CHANGED = "changedFeeds";
/*******************************************************************************
 * ItemsChecker values.
 ******************************************************************************/
const ITEMS_TIMER = 15*DEFAULT_SEC;                           // 15 seconds
const ITEMS_EXPIRES = 5*DEFAULT_MIN;                          // 5 minutes
const ITEMS_EXPIRES_MAX = 2*DEFAULT_HOUR;                     // 2 hours
const ITEMS_VAR = 10*DEFAULT_SEC;                             // 10 seconds
const ITEMS_FAILED = "newslive.loading.failed";
const ITEMS_UPDATE_MAX = 3;
const ITEMS_PREF_COUNT = "itemCount";
/*******************************************************************************
 * NewsliveService values.
 ******************************************************************************/
const SERVICE_DIR = "ProfD";
const SERVICE_FILE = "newslive.sqlite";
const SERVICE_TABLES = {
  feeds: "id TEXT PRIMARY KEY, title TEXT, URL TEXT, iconURL TEXT, " + 
         "refreshRate INTEGER, subscribed BOOLEAN, expiresDate TEXT, " +
         "sortIndex INTEGER",
  items: "id TEXT PRIMARY KEY, feedId TEXT, title TEXT, URL TEXT, " +
         "sortIndex INTEGER"
};
const SERVICE_STMTS = {
  selectFeeds: "SELECT * FROM feeds ORDER BY sortIndex",
  insertFeed: "INSERT INTO feeds (id, title, URL, iconURL, " +
              "refreshRate, subscribed, expiresDate, sortIndex) VALUES " +
              "(:id, :title, :URL, :iconURL, :refreshRate, " +
              ":subscribed, :expiresDate, :sortIndex)",
  updateFeed: "UPDATE feeds set title = :title, URL = :URL, "+
              "iconURL = :iconURL, refreshRate = :refreshRate, " +
              "subscribed = :subscribed, expiresDate = :expiresDate, " +
              "sortIndex = :sortIndex WHERE id = :id",              
  deleteFeed: "DELETE FROM feeds WHERE id = :id",    
  selectItems: "SELECT * FROM items ORDER BY sortIndex",
  insertItem: "INSERT INTO items (id, feedId, title, url, sortIndex) " +
              "VALUES (:id, :feedId, :title, :URL, :sortIndex)",
  updateItem: "UPDATE items SET feedId = :feedId, title = :title, " +
              "URL = :URL, sortIndex = :sortIndex WHERE id = :id",
  deleteItem: "DELETE FROM items WHERE id = :id",
  deleteItems: "DELETE FROM items WHERE feedId = :feedId"
};
const SERVICE_ADD_FEED = "addFeed";
const SERVICE_REMOVE_FEED = "removeFeed";
const SERVICE_UPDATE_FEED = "updateFeed";
const SERVICE_ADD_ITEM = "addItem";
const SERVICE_REMOVE_ITEM = "removeItem";
const SERVICE_UPDATE_ITEM = "updateItem";
const SERVICE_ADD_ITEMS = "addItems";
const SERVICE_REMOVE_ITEMS = "removeItems";
/*******************************************************************************
 * Helper function to create a mozIStorage statement wrappter that can be used 
 * to access the database.
 *
 * @param     conn           The database connection that the statement 
 *                           is applied to.
 * @param     sql            The sql statement that should be created.
 * @return                   The storage wrapper created.
 ******************************************************************************/
function createStatement(conn, sql) {
  var statement = conn.createStatement(sql);
  var rv = Cc[WRAPPER_CONTRACTID].
           createInstance(Ci.mozIStorageStatementWrapper);
  rv.initialize(statement);
  
  return rv;
}
/*******************************************************************************
 * Helper function to create a feed component from a mozIStorage row.
 *
 * @param     row            The row containing the feed data.
 * @return                   The created feed.
 ******************************************************************************/
function createFeedFromRow(row) {
  var rv = Cc[NLF_CONTRACTID].createInstance(Ci.nsINewsliveFeed);
  
  rv.init(row.id, row.title, row.URL, row.iconURL, row.refreshRate, 
          row.subscribed, row.expiresDate, row.sortIndex);

  return rv;
}
/*******************************************************************************
 * Helper function to create an item component from a mozIStorage row.
 *
 * @param     row            The row containing the item data.
 * @return                   The created item.
 ******************************************************************************/
function createItemFromRow(row) {
  var rv = Cc[NLI_CONTRACTID].createInstance(Ci.nsINewsliveItem);
  rv.init(row.id, row.feedId, row.title, row.URL, row.sortIndex);
  return rv;
}
/*******************************************************************************
 * Helper object for doing date time conversions.
 *
 * @param     value          A date value as a millisecond string.
 ******************************************************************************/
function DateUtil(value) {
  this.value = value;
}
DateUtil.prototype = {

  _value: "0",
  get value() { return this._value; },
  set value(val) { this._value = val; },
  
  get date() { return new Date(Number(this.value)); },
  set date(val) { this.value = String(val.getTime()); },  
  
  get time() { return this.date.getTime(); },
  set time(val) { this.date = new Date(val); },
  
  get string() { return this.date.toString(); },
  set string(val) { this.date = new Date(val); },

  get current() { return new Date(); },
  
  // date is less then the current date + a variance
  hasExpired: function du_hasExpired(variance) {
    return (this.time < (this.current.getTime() + variance)); 
  },
  
  // reset the data to current date + value
  reset: function du_reset(value) {
    var time = this.current.getTime() + value;
    var newDate = new Date(time);
    this.date = newDate;
  }
};
/*******************************************************************************
 * Helper function to sort objects by sortIndex.
 ******************************************************************************/
function sorter(a, b) {
  if (a.sortIndex > b.sortIndex)
    return 1;
  else if (a.sortIndex == b.sortIndex)
    return 0;
  else
    return -1;
}
/*******************************************************************************
 * Object used to check the feed list for updates.  This will callback to the
 * Newslive service and set its ready state.
 *
 * @param     service        The newslive service.
 ******************************************************************************/
function FeedsChecker(service) {
  this.nlService = service;
  this.load();
}
FeedsChecker.prototype = {

  _branch: null,
  get branch() { return this._branch; },
  set branch(val) { this._branch = val; },

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _firstTime: null,
  get firstTime() { return this._firstTime; },
  set firstTime(val) { this._firstTime = val; },
  
  _request: null,
  get request() { return this._request; },
  set request(val) { this._request = val; },

  _locked: null,
  get locked() { return this._locked; },
  set locked(val) { this._locked = val; },
  
  _nlService: null,
  get nlService() { return this._nlService; },
  set nlService(val) { this._nlService = val; },

  _ueService: null,
  get ueService() { return this._ueService; },
  set ueService(val) { this._ueService = val; },

  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  get url() { return FEEDS_URL; },
    
  // nsISupports
  QueryInterface: function fc_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function fc_observe(subject, topic, data) { 
    if (topic != TOPIC_TIMER)
      return;
  
    this.fireTimer();
  },
   
  needsUpdated: function fc_needsUpdated(force) {
  
    // feeds are being update we do not need another
    if (this.locked)
      return false;
    
    // force so we update
    if (force)
      return true;
      
    // feeds expired we need an update
    var util = new DateUtil(this.branch.getCharPref("feedsExpire"));
    if (util.hasExpired(FEEDS_VAR))
      return true;
    
    // service has no feeds we need an update  
    if (!this.nlService.hasFeeds())
      return true;
      
    // no update needed
    return false;    
  },
   
  load: function fc_load() {
  
    // get pref branch
    this.branch = Cc[PREF_CONTRACTID].getService(Ci.nsIPrefService).
                  getBranch(DEFAULT_BRANCH);
    
    // get unescape service               
    this.ueService = Cc[UNESCAPE_CONTRACTID].
                     getService(Ci.nsIScriptableUnescapeHTML);    
    
    // get the observer service
    this.obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
      
    // start timer                   
    this.firstTime = true;
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  unload: function fc_unload() {
    // stop any request
    if (this.request)
      this.request.abort();
    
    // stop timer  
    this.timer.cancel();
    
    // remove variables
    this.branch = null;
    this.timer = null;    
    this.firstTime = null;    
    this.request = null;  
    this.locked = null;          
    this.nlService = null;
    this.ueService = null;
    this.obService = null;
  },
  
  reload: function fc_reload() {
    // stop request
    if (this.request)
      this.request.abort();
    this.locked = false;
    
    // request again
    this.requestUpdate(true);
  },
  
  fireTimer: function fc_fireTimer() {
  
    // change timer to a repeating timer if this is the first run
    if (this.firstTime) {
      this.firstTime = false;
      this.timer.init(this, FEEDS_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // request an update
    if (this.needsUpdated())
      this.requestUpdate(false);
  },
      
  requestUpdate: function fc_requestUpdate(force) {
  
    // feed does not need updating
    if (!this.needsUpdated(force))
      return;
    
    // create an XML HTTP request  
    var self = this;
    var request = Cc[REQUEST_CONTRACTID].createInstance(Ci.nsIXMLHttpRequest);
    request.open("GET", this.url, true);
    request.overrideMimeType("text/xml");
    request.setRequestHeader("X-Netscape", "newslive", false);    
    request.onerror = function(event) { self.onError(event);    };
    request.onload = function(event) { self.onLoad(event);     };
    this.request = request;
    
    // send the request
    this.locked = true;
    this.request.send(null);
  },

  onLoad: function fc_onLoad(event) {
    
    // mark loading complete
    this.locked = false;

    // get the list of feeds
    var newFeeds = this.getNewFeeds();
    var state = FEEDS_SAME;
    
    // no feeds in service so add
    if (!this.nlService.hasFeeds()) {
      state = FEEDS_NEW;
      for (var i=0; i<newFeeds.length; i++)
        this.nlService.addFeed(newFeeds[i]);
    
    // merge with existing feeds    
    } else {
      if (this.mergeFeeds(newFeeds))
        state = FEEDS_CHANGED;
    }
      
    // set a new expires date
    var util = new DateUtil("0");
    var max = new DateUtil("0");
    max.reset(FEEDS_EXPIRES_MAX);
    try {
      util.string = this.request.getResponseHeader("Expires");
      if (util.hasExpired(FEEDS_VAR))
        util.reset(FEEDS_EXPIRES);   
      else if (util.time >= max.time)
        util.reset(max.time); 
    } catch(e) {
      util.reset(FEEDS_EXPIRES);
    }
    this.branch.setCharPref("feedsExpire", util.value);
    
    // notify observers
    this.request = null;
    try {
      this.obService.notifyObservers(this.nlService, TOPIC_DEFAULT, state);
    } catch(e) {
      Components.utils.reportError(e);
    }
  },

  onError: function fc_onError(event) {
    this.locked = false;
    this.request = null;
  },
  
  getNewFeeds: function fc_getNewFeeds() {
    // no feeds in document
    if (this.request.responseXML.documentElement.nodeName != "feeds")
      return [];

    // get feed nodes
    var nodes = this.request.responseXML.getElementsByTagName("feed");
    var feeds = [];
    
    // loop through nodes and create feed items
    var subscriptions = this.branch.getCharPref("feedsDefault").split(",");    
    var defaultIndex = subscriptions.length -1;                
    for (var i=0; i<nodes.length; i++) {
    
      var node = nodes[i];
      var id = node.getAttribute("id");
      var title = node.getElementsByTagName("name")[0].textContent;
      title = this._ueService.unescape(title);
      var URL = node.getElementsByTagName("url")[0].textContent;
      var iconURL = node.getElementsByTagName("icon")[0].textContent;
      var refreshRate = Number(node.getAttribute("refreshrate"));
      var index = subscriptions.indexOf(id);
      var subscribed = (index != -1);
      var expiresDate = "0";
      
      ++defaultIndex;
      var sortIndex = (subscribed) ? index : defaultIndex;
      sortIndex = sortIndex*100;
      
      var feed = Cc[NLF_CONTRACTID].createInstance(Ci.nsINewsliveFeed);
      feed.init(id, title, URL, iconURL, refreshRate, subscribed, 
                expiresDate, sortIndex);
      feeds.push(feed);
    }
    
    // return feed items
    return feeds;
  },
  
  mergeFeeds: function fc_mergeFeeds(newFeeds) {
    var rv = false;
    
    // loop through need feeds
    var newFeed;
    var index = newFeeds.length;
    while (index--) {
      newFeed = newFeeds[index];
      
      // add to service if they do not exist
      if (!this.nlService.hasFeed(newFeed.id)) {
        this.nlService.addFeed(newFeed);
        rv = true;
      }
    }
    
    // loop through old feeds
    var oldFeeds = this.nlService.getFeeds(false);
    var oldFeed;
    index = oldFeeds.length;
    while (index--) {
      oldFeed = oldFeeds.queryElementAt(index, Ci.nsINewsliveFeed);
      newFeed = newFeeds.filter(function(o) { return o.id == oldFeed.id; });
      
      // remove if not in the new feed list
      if (newFeed.length === 0) {
        this.nlService.removeFeed(oldFeed.id);
        rv = true;
        
      // update the old feed with new values
      } else {
        newFeed = newFeed.shift();
        oldFeed.init(oldFeed.id, newFeed.title, newFeed.URL, newFeed.iconURL, 
                     newFeed.refreshRate, oldFeed.subscribed, "0", 
                     oldFeed.sortIndex);
        this.nlService.updateFeed(oldFeed); 
      }   
    }    

    // return if the list has changed
    return rv;
  }
};

/*******************************************************************************
 * Object used to check the individual feeds for updates.  This will callback 
 * to the Newslive service and update its data.  This is based on the 
 * places nsLivemarkService.
 *
 * @param     service        The newslive service. 
 ******************************************************************************/
function ItemsChecker(service) {
  this.nlService = service;
  this.load();
}
ItemsChecker.prototype = {

  _branch: null,
  get branch() { return this._branch; },
  set branch(val) { this._branch = val; },
  
  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _firstTime: null,
  get firstTime() { return this._firstTime; },
  set firstTime(val) { this._firstTime = val; },

  _feeds: null,
  get feeds() { return this._feeds; },
  set feeds(val) { this._feeds = val; },

  _failedText: null,
  get failedText() { return this._failedText; },
  set failedText(val) { this._failedText = val; },

  _itemCount: null,
  get itemCount() { return this._itemCount; },
  set itemCount(val) { this._itemCount = val; },
        
  _nlService: null,
  get nlService() { return this._nlService; },
  set nlService(val) { this._nlService = val; },

  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  // nsISupports
  QueryInterface: function ic_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function ic_observe(subject, topic, data) {
    if (topic == TOPIC_TIMER)  
      this.fireTimer();
    else if (topic == TOPIC_PREF) {
      if (data == ITEMS_PREF_COUNT) {
        this.itemCount = this.branch.getIntPref(ITEMS_PREF_COUNT);
        this.reloadAll(); 
      }  
    } else {
      try {
        this[data](subject);    
      } catch(e) {
        Components.utils.reportError(e);
      }
    } 
  },
      
  needsUpdated: function ic_needsUpdated(id, force) {
  
    // feed is loading does not need updated
    if (this.feeds[id].locked)
      return false;
    
    // force so we update
    if (force)
      return true;
    
    // feed expired needs updated    
    var util = new DateUtil(this.feeds[id].expiresDate);
    if (util.hasExpired(ITEMS_VAR))
      return true;
    
    // no items so needs updated
    if (!this.nlService.hasItems(id))
      return true;  
      
    // does not need updated
    return false;  
  },
      
  load: function ic_load() {
  
    // get pref branc add an observer and set the item count
    this.branch = Cc[PREF_CONTRACTID].getService(Ci.nsIPrefService).
                  getBranch(DEFAULT_BRANCH);
    this.branch.QueryInterface(Ci.nsIPrefBranch2).addObserver("", this, false);
    this.itemCount = this.branch.getIntPref(ITEMS_PREF_COUNT);
    
    // create the feeds and failed objects                 
    this.feeds = {};
    this.failedText = this.getString(ITEMS_FAILED) || DEFAULT_FAILED;
    
    // loop through the feeds and add to the checker
    var feeds = this.nlService.getFeeds(true);
    var index = feeds.length;
    while (index--)
      this.pushFeed(feeds.queryElementAt(index, Ci.nsINewsliveFeed)); 
    
    // setup the timer 
    this.firstTime = true;
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    this.timer.init(this, ITEMS_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    
    // add an observer to the service    
    this.obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
    this.obService.addObserver(this, TOPIC_DEFAULT, false);
  },

  unload: function ic_unload() {
  
    // remove the observers
    this.branch.QueryInterface(Ci.nsIPrefBranch2).removeObserver("", this);
    this.obService.removeObserver(this, TOPIC_DEFAULT);
    
    // stop the timer
    this.timer.cancel();
    
    // cancel any pending updates
    var feeds = this.feeds;
    for (var id in feeds) {
      if (feeds[id].loadGroup)
        feeds[id].loadGroup.cancel(Cr.NS_BINDING_ABORTED);
      feeds[id].locked = false;
    }
    
    // remove the variables
    this.branch = null;    
    this.timer = null;
    this.firstTime = null;    
    this.feeds = null;  
    this.failedText = null; 
    this.itemCount = null;    
    this.nlService = null;    
    this.obService = null;
  },
  
  reload: function ic_reload(id) {
    if (!this.feeds.hasOwnProperty(id))
      return;
      
    this.requestUpdate(id, true);
  },
  
  reloadAll: function ic_reloadAll() {
    var feeds = this.sortedFeeds();
    while (feeds.length) {
      var feed = feeds.shift();
      this.requestUpdate(feed.id, true);
    }
  },
         
  fireTimer: function ic_fireTimer() {

    // change timer to a repeating timer if this is the first run  
    if (this.firstTime) {
      this.firstTime = false;
      this.timer.init(this, ITEMS_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // loop through the feeds and update
    var count = ITEMS_UPDATE_MAX;
    var feeds = this.sortedFeeds();
    while (feeds.length) {
      var feed = feeds.shift();
      if (this.requestUpdate(feed.id, false))
        count--;
        
      // only update the maximum number of feeds at a time
      if (!count)
        break;
    }
  },
  
  requestUpdate: function ic_requestUpdate(id, force) {
  
    // feed does not need updating
    if (!this.needsUpdated(id, force))
      return false;
      
    // lock the feed and remove it's items
    feed = this.feeds[id];  
    feed.locked = true;
    this.nlService.removeItems(feed.id);
    
    var loadGroup;
    try {
      // setup the loadgroup      
      var ioService = Cc[IO_CONTRACTID].getService(Ci.nsIIOService);
      loadGroup = Cc[LOAD_CONTRACTID].createInstance(Ci.nsILoadGroup);
      
      // set the channel flags
      var uriChannel = ioService.newChannel(feed.URL, null, null);
      uriChannel.loadGroup = loadGroup;
      uriChannel.loadFlags |= Ci.nsIRequest.LOAD_BACKGROUND ||
                              Ci.nsIRequest.VALIDATE_ALWAYS;

      // set the request header
      var httpChannel = uriChannel.QueryInterface(Ci.nsIHttpChannel);
      httpChannel.requestMethod = "GET";
      httpChannel.setRequestHeader("X-Netscape", "newslive", false);
      
      // open the request
      var listener = new ItemsListener(feed, this);
      httpChannel.asyncOpen(listener, null);
    
    // setup failed  
    } catch(e) {
      feed.locked = false;
      this.insertFailed(feed);
      Components.utils.reportError(e);
      return false;
    }
    
    // set the loadgroup and return success
    feed.loadGroup = loadGroup; 
    return true;
  },
    
  pushFeed: function ic_pushFeed(feed) {
    this.feeds[feed.id] = { id: feed.id, URL: feed.URL, refreshRate: 
                             feed.refreshRate, expiresDate: feed.expiresDate,
                             sortIndex: feed.sortIndex };
  },
  
  sortedFeeds: function ic_sortedFeeds() {
    var rv = [];
    var feeds = this.feeds;
    for (var id in feeds)
      rv.push(feeds[id]);
    rv.sort(sorter);
    return rv; 
  },
  
  getString: function ic_getString(name) {
    var sbService = Cc[BUNDLE_CONTRACTID].getService(Ci.nsIStringBundleService);
    var bundle = sbService.createBundle(DEFAULT_BUNDLE);
    if (bundle)
      return bundle.GetStringFromName(name);
    return null;
  },
    
  insertFailed: function ic_insertFailed(feed) {
    // remove the items from the service
    this.nlService.removeItems(feed.id);  
    var item = Cc[NLI_CONTRACTID].createInstance(Ci.nsINewsliveItem);
    
    // insert the failed item
    var id = feed.id + "_failed";
    item.init(id, feed.id, this.failedText, "about:newslive", false, 0);
    this.nlService.addItem(item);
  },
  
  sameFeeds: function ic_sameFeeds(service) {},
  newFeeds: function ic_newFeeds(service) {},
  changedFeeds: function ic_changedFeeds(service) {},

  addFeed: function ic_addFeed(feed) { 
    feed = feed.QueryInterface(Ci.nsINewsliveFeed);

    // save the feed if we subscribe to it
    if (feed.subscribed)
      this.pushFeed(feed);
  },
  
  removeFeed: function ic_removeFeed(feed) { 
    feed = feed.QueryInterface(Ci.nsINewsliveFeed);  
    
    // we do not have the feed so ignore  
    if (!this.feeds.hasOwnProperty(feed.id))
      return;
    
    // cancel the load
    var obj = this.feeds[feed.id];
    if (obj.loadGroup) {
      obj.loadGroup.cancel(Cr.NS_BINDING_ABORTED);
      obj.locked = false;
    }
    
    // delete the feed
    delete this.feeds[feed.id];    
  },
  
  updateFeed: function ic_updateFeed(feed) { 
    feed = feed.QueryInterface(Ci.nsINewsliveFeed);
    
    // remove the feed if not subscribed
    if (!feed.subscribed)
      this.removeFeed(feed);
      
    // save the feed if subscribed
    else
      this.pushFeed(feed);
  },
  
  addItem: function ic_addItem(item) {},
  removeItem: function ic_removeItem(item) {},
  updateItem: function ic_updateItem(item) {},
  addItems: function ic_addItems(items) {},
  removeItems: function ic_removeItems(feed) {}
};

/*******************************************************************************
 * Object used to listen for channel progress and parse the feed from the
 * channel.
 *
 * @param     feed           An object hold the feed information from the
 *                           items checker.
 * @param     service        The newslive service. 
 ******************************************************************************/
function ItemsListener(feed, checker) {
  this.feed = feed;
  this.checker = checker;
  this.isError = false;
  this.processor = null;
}
ItemsListener.prototype = {

  _isError: null, 
  get isError() { return this._isError; },
  set isError(val) { this._isError = val; },
  
  _feed: null,
  get feed() { return this._feed; },
  set feed(val) { this._feed = val; },
  
  _checker: null,
  get checker() { return this._checker; },
  set checker(val) { this._checker = val; },
  
  _processor: null,
  get processor() { return this._processor; },
  set processor(val) { this._processor = val; },
    
  // nsISupports
  QueryInterface: function il_QI(iid) {
    if (iid.equals(Ci.nsIFeedResultListener) ||
        iid.equals(Ci.nsIStreamListener) ||
        iid.equals(Ci.nsIRequestObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIFeedResultListener
  handleResult: function il_handleResult(result) {
    
    // there was an error in loading just return
    var feed = this.feed;
    if (this.isError) {
      feed.locked = false;
      return;
    }
    
    try {
    
      // remove the items from the service
      this.checker.nlService.removeItems(feed.id);
      
      // invalid feed just return
      if (!result || !result.doc || result.bozo) {
        this.checker.insertFailed(feed);
        feed.locked = false;
        return;
      }
      
      // loop through the feed entries    
      var title, url, entry;
      var items = Cc[ARRAY_CONTRACTID].createInstance(Ci.nsIMutableArray);
      var iface = result.doc.QueryInterface(Ci.nsIFeed);
      
      for (var i=0; i< iface.items.length; i++) {
      
        // max items hit leave loop
        if (i >= this.checker.itemCount)
          break;
        
        // get the entry title  
        entry = iface.items.queryElementAt(i, Ci.nsIFeedEntry);
        if (entry.title)
          title = entry.title.plainText();
        else if (entry.updated)
          title = entry.updated;
         
        // get the entry uri
        if (entry.link)
          uri = entry.link;

        // create the item
        if (uri && title)
          items.appendElement(this.createItem(uri, title, i*100), false);    
      }
      
      // add the items to the service
      if (items.length > 0)
        this.checker.nlService.addItems(items.QueryInterface(Ci.nsIArray));
    
    // cleanup the listener    
    } finally {
      this.processor.listener = null;
      this.processor = null;
      feed.locked = false;
    }
  },

  // nsIStreamListener
  onDataAvailable: function il_onDataAvailable(request, context, 
                                                          stream, offset, 
                                                          count) {                                             
      this.processor.onDataAvailable(request, context, stream, offset, count);
  },
  
  // nsIRequestObserver
  onStartRequest: function il_onStartRequest(request, context) {
    if (this.isError)
      throw Cr.NS_ERROR_UNEXPECTED;
    
    // create a feed processor
    var channel = request.QueryInterface(Ci.nsIChannel);
    this.processor = Cc[FPR_CONTRACTID].createInstance(Ci.nsIFeedProcessor);
    this.processor.listener = this;
    
    // start processing
    this.processor.parseAsync(null, channel.URI);
    this.processor.onStartRequest(request, context);               
  },
  
  onStopRequest: function il_onStopRequest(request, context, status) {
  
    // mark that an error occurred if not successful
    if (!Components.isSuccessCode(status)) {
      this.isError = true;
      this.setExpires(ITEMS_EXPIRES);
      this.checker.insertFailed(this.feed);
      return;
    }
    
    // stop the processor and set the expiration date
    try {      
      this.processor.onStopRequest(request, context, status);
      this.setExpires(null);
    } catch(e) {
      this.setExpires(ITEMS_EXPIRES);
    } 
  },                                                          
   
  setExpires: function il_setExpires(duration) {
    var feed = this.checker.nlService.getFeed(this.feed.id);
    if (!feed)
      return;
        
    var rate = (!duration) ? this.feed.refreshRate*DEFAULT_SEC : duration; 
    var util = new DateUtil("0");
    var max = new DateUtil("0");
    max.reset(ITEMS_EXPIRES_MAX);
    util.reset(rate);
    if (util.time > max.time)
      util.reset(max.time);
          
    feed.expiresDate = util.value;
    this.checker.nlService.updateFeed(feed);
  },
  
  createItem: function il_createItemItem(uri, title, index) {
    var feed = this.feed;
    var item = Cc[NLI_CONTRACTID].createInstance(Ci.nsINewsliveItem);
    var id = feed.id + "_" + index;
    item.init(id, feed.id, title, uri.spec, index);
    
    return item;
  }
};

/*******************************************************************************
 * Interface used to describe a news live feed.  This interface is used to 
 * represent meta data describing a netscape news feed.
 * 
 * @version   1.0
 ******************************************************************************/
function NewsliveFeed() {}
NewsliveFeed.prototype = {
  _id: null,  
  _title: null,
  _URL: null,
  _iconURL: null,
  _refreshRate: null,
  _subscribed: null,
  _expiresDate: null,
  _sortIndex: null,
  
  // nsISupports
  QueryInterface: function nlf_QI(iid) {
    if (iid.equals(Ci.nsINewsliveFeed) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
    
  // nsINewsliveFeed
  get id() { return this._id; },
  get title() { return this._title; },
  get URL() { return this._URL; },
  get iconURL() { return this._iconURL; },
  get refreshRate() { return this._refreshRate; },
  get subscribed() { return this._subscribed; },
  get expiresDate() { return this._expiresDate; },
  get sortIndex() { return this._sortIndex; },
  
  set subscribed(val) { this._subscribed = val; },
  set expiresDate(val) { this._expiresDate = val; },
  set sortIndex(val) { this._sortIndex = val; },
  
  init: function nlf_init(id, title, URL, iconURL, 
                                   refreshRate, subscribed, expiresDate, 
                                   sortIndex) {
    this._id = id;
    this._title = title;
    this._URL = URL;
    this._iconURL = iconURL;
    this._refreshRate = refreshRate;
    this._subscribed = subscribed;
    this._expiresDate = expiresDate;
    this._sortIndex = sortIndex;
  },

  clone: function nlf_clone() {
    var rv = Cc[NLF_CONTRACTID].createInstance(Ci.nsINewsliveFeed);

    rv.init(this._id, this._title, this._URL, this._iconURL,
            this._refreshRate, this._subscribed, this._expiresDate, 
            this._sortIndex);
    
    return rv;
  }          
};
/*******************************************************************************
 * Interface used to describe a news live feed item.  This interface is used to 
 * represent meta data describing a netscape news feed item.
 * 
 * @version   1.0
 ******************************************************************************/
function NewsliveItem() {}
NewsliveItem.prototype = {
  _id: null,
  _feedId: null,
  _title: null,
  _URL: null,
  _sortIndex: null,
  
  // nsISupports
  QueryInterface: function nli_QI(iid) {
    if (iid.equals(Ci.nsINewsliveItem) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },

  // nsINewsliveItem
  get id() { return this._id; }, 
  get feedId() { return this._feedId; },  
  get title() { return this._title; }, 
  get URL() { return this._URL; },
  get sortIndex() { return this._sortIndex; },
  
  set sortIndex(val) { this._sortIndex = val; },
  
  init: function nli_init(id, feedId, title, URL, sortIndex) {
    this._id = id;
    this._feedId = feedId;
    this._title = title;
    this._URL = URL;
    this._sortIndex = sortIndex;
  },

  clone: function nli_clone() {
    var rv = Cc[NLI_CONTRACTID].createInstance(Ci.nsINewsliveItem);

    rv.init(this._id, this._feedId, this._title,
            this._URL, this._sortIndex);

    return rv;
  }     
};

/*******************************************************************************
 * The main news live interface.  This interface takes care of updating the 
 * list of feeds and the individual feed items.  It notifies the front-end
 * when updates take place.
 * 
 * @version   1.0
 ******************************************************************************/
function NewsliveService() {}
NewsliveService.prototype = {
  _feeds: null,
  _items: null,
  _conn: null,
  _statements: null,  
  _feedsChecker: null,
  _itemsChecker: null,
  _obService: null,
        
  // nsISupports
  QueryInterface: function nls_QI(iid) {
    if (iid.equals(Ci.nsINewsliveService) ||
        iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function nls_observe(subject, topic, data) {
    switch (topic) {
          
    case TOPIC_STARTUP_APP:
      this._load();
      break;
    
    case TOPIC_STARTUP_UI:
      this._loadDelayed();
      break;
    
    case TOPIC_SHUTDOWN_APP:
      this._unload();
      break;
              
    case TOPIC_SHUTDOWN_XPCOM:
      this._unloadDelayed();
      break;
    
    default:
      return;
    }
  },
  
  // nsINewsliveService 
  get databaseFile() { 
    if (!this._conn) 
      return null; 
    return this._conn.databaseFile; 
  },
    
  hasFeed: function nls_hasFeed(id) { 
    return this._feeds.some(function(o) { return o.id == id; });
  },
  
  hasFeeds: function nls_hasFeeds() {
    return this._feeds.length > 0;
   },
  
  getFeed: function nls_getFeed(id) {
    if (!this.hasFeed(id))
      return null;
    return this._feeds.filter(function(o) { return o.id == id; }).shift();
  },
  
  getFeeds: function nls_getFeeds(subscribed) { 
    var rv = Cc[ARRAY_CONTRACTID].createInstance(Ci.nsIMutableArray);      
    
    function check(o) {
      if (!subscribed || (subscribed && o.subscribed))
        return true;
      return false;
    }
    
    var feeds = this._feeds.filter(check);
    feeds.sort(sorter);
    
    var index = 0;
    while (index < feeds.length) {
      rv.appendElement(feeds[index], false);
      index++;
    }
    
    return rv.QueryInterface(Ci.nsIArray);
  },
    
  addFeed: function nls_addFeed(feed) { 
    if (this.hasFeed(feed.id)) {
      this.updateFeed(feed);
      return;
    }
    
    this._feeds.push(feed.clone());

    this._conn.beginTransaction();
    var statement = this._statements["insertFeed"];
    var params = statement.params;            
    params.id = feed.id;
    params.title = feed.title;
    params.URL = feed.URL;
    params.iconURL = feed.iconURL;
    params.refreshRate = feed.refreshRate;
    params.subscribed = feed.subscribed;
    params.expiresDate = feed.expiresDate; 
    params.sortIndex = feed.sortIndex;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(feed, SERVICE_ADD_FEED);
  },
  
  updateFeed: function nls_updateFeed(feed) { 
    if (!this.hasFeed(feed.id))
      return;
  
    var feeds = this._feeds.filter(function(o) { return o.id != feed.id; });
    feeds.push(feed.clone());
    this._feeds = feeds;
    
    this._conn.beginTransaction();
    var statement = this._statements["updateFeed"];
    var params = statement.params;
    params.id = feed.id;
    params.title = feed.title;
    params.URL = feed.URL;
    params.iconURL = feed.iconURL;
    params.refreshRate = feed.refreshRate;
    params.subscribed = feed.subscribed;
    params.expiresDate = feed.expiresDate; 
    params.sortIndex = feed.sortIndex;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(feed, SERVICE_UPDATE_FEED);  
  },
    
  removeFeed: function nls_removeFeed(id) { 
    var feed = this.getFeed(id);
    if (!feed)
      return;
    
    var feeds = this._feeds.filter(function(o) { return o.id != id; });
    this._feeds = feeds;
    
    this._conn.beginTransaction();
    var statement = this._statements["deleteFeed"];
    var params = statement.params;
    params.id = feed.id;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(feed, SERVICE_REMOVE_FEED);      
  },
  
  hasItem: function nls_hasItem(id) { 
    return this._items.some(function(o) { return o.id == id; });
  },
  
  hasItems: function nls_hasItems(feedId) { 
    return this._items.some(function(o) { return ((o.feedId == feedId) && 
                                          (o.URL != "about:newslive")); });
  },
  
  getItem: function nls_getItem(id) {
    if (!this.hasItem(id))
      return null;  
    return this._items.filter(function(o) { return o.id == id; }).shift();
  },
    
  getItems: function nls_getItems(feedId) { 
    var rv = Cc[ARRAY_CONTRACTID].createInstance(Ci.nsIMutableArray);      
     
    var items = this._items.filter(function(o) { return o.feedId == feedId; });
    items.sort(sorter);
    
    var index = 0;
    while (index < items.length) {
      rv.appendElement(items[index], false);
      index++;
    }
    
    return rv.QueryInterface(Ci.nsIArray);
  },
  
  addItem: function nls_addItem(item) { 
    if (this.hasItem(item.id)) {
      this.updateItem(item);
      return;
    }

    this._items.push(item.clone());
    
    this._conn.beginTransaction();
    var statement = this._statements["insertItem"];
    var params = statement.params;
    params.id = item.id;
    params.feedId = item.feedId;
    params.title = item.title;
    params.URL = item.URL;
    params.sortIndex = item.sortIndex;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(item, SERVICE_ADD_ITEM);
  },
    
  addItems: function nls_addItems(items) {

    var index = 0;
    var statement, params;
    this._conn.beginTransaction();         
    while (index < items.length) {
      var item = items.queryElementAt(index, Ci.nsINewsliveItem);
      
      if (this.hasItem(item.id)) {
        var filtered = this._items.filter(function(o) { return o.id != item.id; });
        this._items = filtered;

        statement = this._statements["deleteItem"];
        params = statement.params;
        params.id = item.id;
        statement.execute();
        this._notify(items, SERVICE_REMOVE_ITEM);
      }      

      this._items.push(item.clone());
      
      statement = this._statements["insertItem"];
      params = statement.params;
      params.id = item.id;
      params.feedId = item.feedId;
      params.title = item.title;
      params.URL = item.URL;
      params.sortIndex = item.sortIndex;
      statement.execute();
      index++;
    }
    this._conn.commitTransaction(); 
    
    this._notify(items, SERVICE_ADD_ITEMS);
  },
   
  updateItem: function nls_updateItem(item) { 
    if (!this.hasItem(item.id))
      return;
  
    var items = this._items.filter(function(o) { return o.id != item.id; });
    items.push(item.clone());
    this._items = items;
    
    this._conn.beginTransaction();
    var statement = this._statements["updateItem"];
    var params = statement.params;
    params.id = item.id;
    params.feedId = item.feedId;
    params.title = item.title;
    params.URL = item.URL;
    params.sortIndex = item.sortIndex;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(item, SERVICE_UPDATE_ITEM);
  },
    
  removeItem: function nls_removeItem(id) {
    var item = this.getItem(id);
    if (!item)
      return;

    var items = this._items.filter(function(o) { return o.id != item.id; });
    this._items = items; 
    
    this._conn.beginTransaction();
    var statement = this._statements["deleteItem"];
    var params = statement.params;
    params.id = item.id;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(item, SERVICE_REMOVE_ITEM);      
  },
  
  removeItems: function nls_removeItems(feedId) { 
    var feed = this.getFeed(feedId);
    if (!feed)
      return;
    
    var items = this._items.filter(function(o) { return o.feedId != feedId; });
    this._items = items; 

    this._conn.beginTransaction();
    var statement = this._statements["deleteItems"];
    var params = statement.params;
    params.feedId = feedId;
    statement.execute();
    this._conn.commitTransaction();
    
    this._notify(feed, SERVICE_REMOVE_ITEMS);      
  },
  
  reloadFeeds: function nls_reloadFeeds() {
    this._feedsChecker.reload();
  },
  
  reloadItems: function nls_reloadItems(feedId) {
    this._itemsChecker.reload(feedId);
  },

  compactDB: function nls_compactDB() {
    if (this._conn.transactionInProgress)
      throw Cr.NS_ERROR_FAILURE;    
    this._conn.executeSimpleSQL("VACUUM");  
  },
        
  _load: function nls__load() {
    this._obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
    this._obService.addObserver(this, TOPIC_STARTUP_UI, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_APP, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_XPCOM, false);
    
    this._feeds = [];
    this._items = [];
  },
  
  _loadDelayed: function nls__loadDelayed() {
    this._loadDB();
    
    this._feedsChecker = new FeedsChecker(this);
    this._itemsChecker = new ItemsChecker(this);  
  },
  
  _unload: function nls__unload() {
    this._feedsChecker.unload();
    this._itemsChecker.unload();
  },
  
  _unloadDelayed: function nls__unloadDelayed() {
         
    // remove observers
    this._obService.removeObserver(this, TOPIC_STARTUP_UI);
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_APP);    
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_XPCOM);

    // remove variables
    this._feeds = null;
    this._items = null;
    this._conn = null;        
    this._statements = null;
    this._feedsChecker = null;
    this._itemsChecker = null;
    this._obService = null;
  },
  
  _loadDB: function nls__loadDB() {
  
    // get the database file
    var dir = Cc[DIR_CONTRACTID].getService(Ci.nsIProperties);
    var file = dir.get(SERVICE_DIR, Ci.nsIFile);    
    file.append(SERVICE_FILE);
    
    // open a connection to the database
    var storage = Cc[STORAGE_CONTRACTID].getService(Ci.mozIStorageService);
    try {
      this._conn = storage.openDatabase(file);
    } catch(e) {
      
      // database is corrupt - remove and try again
      if (e.result == 0x805200b) {
        file.remove(false);
        this._conn = storage.openDatabase(file);
        
      // unknown error  
      } else
        throw e;
    }
    
    // create the tables
    for (var name in SERVICE_TABLES) {
      if (!this._conn.tableExists(name))
        this._conn.createTable(name, SERVICE_TABLES[name]);
    }
    
    // create the needed statements
    this._statements = {};
    for (name in SERVICE_STMTS)
      this._statements[name] = createStatement(this._conn, SERVICE_STMTS[name]);       
    
    // load the cache
    var statement = this._statements["selectFeeds"];
    while (statement.step())
      this._feeds.push(createFeedFromRow(statement.row));  
    statement.reset();
    statement = this._statements["selectItems"];
    while (statement.step())
      this._items.push(createItemFromRow(statement.row));  
    statement.reset();    
  },
  
  _notify: function nls__notifyObservers(subject, data) {
    try {
      this._obService.notifyObservers(subject, TOPIC_DEFAULT, data);
    } catch(e) {
      Components.utils.reportError(e);
    }
  }
};
/*******************************************************************************
 * Generic object used as a factory based on a passed in constructor.
 ******************************************************************************/
function GenericFactory(ctor) {
  this._ctor = ctor;
}  
GenericFactory.prototype = {
  _ctor: null,
  
  // nsISupports
  QueryInterface: function gf_QI(iid) {
    if (iid.equals(Ci.nsIFactory) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIFactory
  createInstance: function gf_createInstance(outer, iid) {
    if (outer !== null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return (new this._ctor()).QueryInterface(iid);
  },

  lockFactory: function gf_lockFactory(loc) {
    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
  }
};
/*******************************************************************************
 * Object implementing the nsIModule interface.  This is used to retrieve 
 * the different components and services the module implements.
 ******************************************************************************/
var Module = {

  // nsISupports
  QueryInterface: function mod_QI(iid) {
    if (iid.equals(Ci.nsIModule) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  
  // nsIModule
  getClassObject: function mod_getClassObject(cm, cid, iid) {
    if (!iid.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    
    if (cid.equals(NLF_CLASSID))
      return new GenericFactory(NewsliveFeed);
    else if (cid.equals(NLI_CLASSID))
      return new GenericFactory(NewsliveItem);
    else if (cid.equals(NLS_CLASSID))
      return new GenericFactory(NewsliveService);
          
    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  registerSelf: function mod_registerSelf(cm, file, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);  
    
    cr.registerFactoryLocation(NLF_CLASSID, NLF_CLASSNAME, NLF_CONTRACTID,
                               file, location, type);
    cr.registerFactoryLocation(NLI_CLASSID, NLI_CLASSNAME, NLI_CONTRACTID,
                               file, location, type);
    cr.registerFactoryLocation(NLS_CLASSID, NLS_CLASSNAME, NLS_CONTRACTID,
                               file, location, type);   
                                
    var catMgr = Components.classes[CAT_CONTRACTID].
                 getService(Ci.nsICategoryManager);
    catMgr.addCategoryEntry(TOPIC_STARTUP_APP, NLS_CLASSNAME, "service," + 
                            NLS_CONTRACTID, true, true);                                                                                         
  },

  unregisterSelf: function mod_unregisterSelf(cm, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);
    
    cr.unregisterFactoryLocation(NLF_CLASSID, location);
    cr.unregisterFactoryLocation(NLI_CLASSID, location);
    cr.unregisterFactoryLocation(NLS_CLASSID, location);
                                
    var catMgr = Components.classes[CAT_CONTRACTID].
                 getService(Ci.nsICategoryManager); 
    catMgr.deleteCategoryEntry(TOPIC_STARTUP_APP, "service," + 
                               NLS_CONTRACTID, true);
  },

  canUnload: function mod_canUnload(cm) { return true; }
};
/*******************************************************************************
 * Entry point used to retrieve the module object.
 ******************************************************************************/
function NSGetModule(cm, file) { return Module; }